// Author: Gaston C. Hillar
// Oracle Java Magazine
import mraa.Aio;
import mraa.Pwm;


class VariableBrightnessLed {
    private final int gpioPin;
    private final String name;
    private final Pwm pwm;
    private int brightnessLevel;
        
    
    public VariableBrightnessLed(int gpioPin, 
        String name) {
        this.name = name;
        this.gpioPin = gpioPin;
        this.pwm = new Pwm(gpioPin);
        this.pwm.period_us(700);
        this.pwm.enable(true);
        // Set the initial brightness level to 0
        this.setBrightnessLevel(0);
    }
    
    public void setBrightnessLevel(
        int brightnessLevel) {
        int validBrightnessLevel = brightnessLevel;
        if (validBrightnessLevel > 255) {
            validBrightnessLevel = 255;
        } else if (validBrightnessLevel < 0){
            validBrightnessLevel = 0;
        }
        float convertedLevel = 
            validBrightnessLevel / 255f;
        this.pwm.write(convertedLevel);
        this.brightnessLevel = validBrightnessLevel;
        System.out.format(
            "%s LED connected to PWM Pin #%d " +
            "set to brightness level %d.%n",
            this.name, 
            this.gpioPin, 
            validBrightnessLevel);
    }
    
    public int getBrightnessLevel() {
        return this.brightnessLevel;
    }
    
    public int getGpioPin() {
        return this.gpioPin;
    }
}

class VoltageInput {
    private final int analogPin;
    private final Aio aio;
    
    public VoltageInput(int analogPin) {
        this.analogPin = analogPin;
        this.aio = new Aio(analogPin);
        // Configure the ADC 
        // (short for Analog-to-Digital Converter)
        // resolution to 12 bits (0 to 4095)
        this.aio.setBit(12);
    }
    
    public float getVoltage() {
        long raw_value = this.aio.read();
        float voltage_value = 
            raw_value / 4095f * 5f;
        return voltage_value;
    }

    public int getAnalogPin() {
        return analogPin;
    }
}

class SimpleLightSensor {
    // Light level descriptions
    public static final String 
        LL_EXTREMELY_DARK = "Extremely dark";
    public static final String 
        LL_VERY_DARK = "Very dark";
    public static final String 
        LL_JUST_DARK = "Just dark";
    public static final String 
        LL_SUNNY_DAY = "Like a sunny day";
    // Maximum voltages that determine the light level
    private static final float 
        EXTREMELY_DARK = 2.1f;
    private static final float 
        VERY_DARK = 3.1f;
    private static final float 
        JUST_DARK = 4.05f;
    
    private final VoltageInput voltageInput;
    private float measuredVoltage = 0f;
    private String lightLevel = 
        SimpleLightSensor.LL_SUNNY_DAY;
    
    public SimpleLightSensor(int analogPin) {
        this.voltageInput = 
            new VoltageInput(analogPin);
        this.measureDarkness();
    }
    
    public void measureDarkness() {
        this.measuredVoltage = 
            this.voltageInput.getVoltage();
        if (this.measuredVoltage < 
            SimpleLightSensor.EXTREMELY_DARK) {
            this.lightLevel = 
                SimpleLightSensor.LL_EXTREMELY_DARK;
        } else if (this.measuredVoltage < 
            SimpleLightSensor.VERY_DARK) {
            this.lightLevel = 
                SimpleLightSensor.LL_VERY_DARK;
        } else if (this.measuredVoltage < 
            SimpleLightSensor.JUST_DARK) {
            this.lightLevel = 
                SimpleLightSensor.LL_JUST_DARK;
        } else {
            this.lightLevel = 
                SimpleLightSensor.LL_SUNNY_DAY;
        }
    }
    
    public String getLightLevel() {
        return this.lightLevel;
    }
}

class BoardManager {
    public final SimpleLightSensor lightSensor;
    public final VariableBrightnessLed redLed;
    public final VariableBrightnessLed greenLed;
    public final VariableBrightnessLed blueLed;
    
    public BoardManager() {
        // The photoresistor included in the 
        // voltage divider is connected to 
        // the analog PIN A0
        this.lightSensor = 
            new SimpleLightSensor(0);
        // The Red LED is connected to GPIO pin ~6
        this.redLed = 
            new VariableBrightnessLed(6, "Red");
        // The Green LED is connected to GPIO Pin ~5
        this.greenLed = 
            new VariableBrightnessLed(5, "Green");
        // The Blue LED is connected to GPIO Pin ~3
        this.blueLed = 
            new VariableBrightnessLed(3, "Blue");
    }
    
    public void setRGBBrightnessLevel(int value) {
        this.redLed.setBrightnessLevel(value);
        this.greenLed.setBrightnessLevel(value);
        this.blueLed.setBrightnessLevel(value);
    }
    
    public void updateLedsBasedOnLight() {
        String lightLevel = 
            this.lightSensor.getLightLevel();
        switch (lightLevel) {
            case SimpleLightSensor.LL_EXTREMELY_DARK:
                this.setRGBBrightnessLevel(255);
                break;
            case SimpleLightSensor.LL_VERY_DARK:
                this.setRGBBrightnessLevel(128);
                break;
            case SimpleLightSensor.LL_JUST_DARK:
                this.setRGBBrightnessLevel(64);
                break;
            default:
                this.setRGBBrightnessLevel(0);
                break;
        }
    }
}

public class AmbientLightAndLed {

    public static void main(String[] args) {
        String lastlightLevel = "";
        BoardManager board = new BoardManager();
        while (true) {
            board.lightSensor.measureDarkness();
            String newlightLevel = 
                board.lightSensor.getLightLevel();
            if (newlightLevel != lastlightLevel) {
                // The measured light level has changed
                lastlightLevel = newlightLevel;
                System.out.format(
                    "Measured light level: %s%n", 
                    newlightLevel);
                board.updateLedsBasedOnLight();
            }
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                System.err.format(
                    "Sleep interruption: %s",
                    e.toString());
            }
        }
    }
}
